import { MatchRequest } from '../domain/dtos/match.request';
import { Geography } from '../domain/entities/ecosystem/geography';
import { Ad } from '../domain/entities/ecosystem/ad';
import { Requirement } from '../domain/entities/ecosystem/requirement';
import { Role } from '../domain/types/role.enum';
import { AlgorithmSettings } from '../domain/entities/ecosystem/algorithm-settings';
import { Time } from '../domain/entities/ecosystem/time';
import { IDefaultParams } from '../domain/types/default-params.type';
import { IGeorouter } from '../domain/interfaces/georouter.interface';
import { ICreateGeorouter } from '../domain/interfaces/georouter-creator.interface';
import { IFindTimezone } from '../../geography/domain/interfaces/timezone-finder.interface';
import { Mode } from '../domain/types/mode.enum';
import { IConvertTime } from '../domain/interfaces/time-converter.interface';

export class MatchQuery {
  private readonly matchRequest: MatchRequest;
  private readonly defaultParams: IDefaultParams;
  private readonly georouterCreator: ICreateGeorouter;
  mode: Mode;
  ad: Ad;
  roles: Role[];
  time: Time;
  geography: Geography;
  exclusions: string[];
  requirement: Requirement;
  algorithmSettings: AlgorithmSettings;
  georouter: IGeorouter;
  timezoneFinder: IFindTimezone;
  timeConverter: IConvertTime;

  constructor(
    matchRequest: MatchRequest,
    defaultParams: IDefaultParams,
    georouterCreator: ICreateGeorouter,
    timezoneFinder: IFindTimezone,
    timeConverter: IConvertTime,
  ) {
    this.matchRequest = matchRequest;
    this.defaultParams = defaultParams;
    this.georouterCreator = georouterCreator;
    this.timezoneFinder = timezoneFinder;
    this.timeConverter = timeConverter;
    this.setMode();
    this.setAd();
    this.setRoles();
    this.setGeography();
    this.setTime();
    this.setRequirement();
    this.setAlgorithmSettings();
    this.setExclusions();
  }

  createRoutes = (): void => {
    this.geography.createRoutes(this.roles, this.algorithmSettings.georouter);
  };

  private setMode = (): void => {
    this.mode = this.matchRequest.mode ?? Mode.MATCH;
  };

  private setAd = (): void => {
    this.ad = new Ad(
      this.matchRequest,
      this.defaultParams.DEFAULT_UUID,
      this.defaultParams.MARGIN_DURATION,
    );
    this.ad.init();
  };

  private setRoles = (): void => {
    this.roles = [];
    if (this.matchRequest.driver) this.roles.push(Role.DRIVER);
    if (this.matchRequest.passenger) this.roles.push(Role.PASSENGER);
    if (this.roles.length == 0) this.roles.push(Role.PASSENGER);
  };

  private setGeography = (): void => {
    this.geography = new Geography(
      this.matchRequest,
      {
        timezone: this.defaultParams.DEFAULT_TIMEZONE,
        finder: this.timezoneFinder,
      },
      this.ad,
    );
    this.geography.init();
    if (this.geography.timezones.length > 0)
      this.matchRequest.timezone = this.geography.timezones[0];
  };

  private setTime = (): void => {
    this.time = new Time(
      this.matchRequest,
      this.defaultParams.MARGIN_DURATION,
      this.defaultParams.VALIDITY_DURATION,
    );
    this.time.init();
  };

  private setRequirement = (): void => {
    this.requirement = new Requirement(
      this.matchRequest,
      this.defaultParams.DEFAULT_SEATS,
    );
  };

  private setAlgorithmSettings = (): void => {
    this.algorithmSettings = new AlgorithmSettings(
      this.matchRequest,
      this.defaultParams.DEFAULT_ALGORITHM_SETTINGS,
      this.time.frequency,
      this.georouterCreator,
    );
  };

  private setExclusions = (): void => {
    this.exclusions = [];
    if (this.matchRequest.uuid) this.exclusions.push(this.matchRequest.uuid);
    if (this.matchRequest.exclusions)
      this.exclusions.push(...this.matchRequest.exclusions);
  };
}
